
/*
 "Help for the elderly app"

We have created an app that helps the elderly to manage day to day life and alerts their caregiver to an emergency.
The app helps elderly in several scenarios:
    - Manages the elderly person's medication and shows him when he needs to take medications and the number of pills in the app.
    - Get weather news and get suggestions in case of a rainy day
    - Check the temperature of the room. In case of extremely high temperature, it will alert.
  In all those three scenarios the family members,  get a message to their phones with a telegram notification.
  
  The circuit:
  Input  - temperature (thermometer)/
  Output - red leds (to alert for high temperature or when the baby is crying)
           and via Blynk and Integromat Android and Telegram messages.
*/

#define BLYNK_TEMPLATE_ID "TMPLYFvKdF-g"
#define BLYNK_DEVICE_NAME "Final project"
#define BLYNK_AUTH_TOKEN "C5bOr7BxznzCsWY2YCSAjivlW0wuUdOD"

#define BLYNK_PRINT SerialUSB

#include <Adafruit_NeoPixel.h>
#define LED_PIN    8
#define LED_COUNT 10
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);


#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <Adafruit_CircuitPlayground.h>

char auth[] = BLYNK_AUTH_TOKEN;

char ssid[] = "YOUR_WIFI";
char pass[] = "WIFI_PASSWORD";

#define EspSerial Serial1

// Your ESP8266 baud rate:
#define ESP8266_BAUD 115200

ESP8266 wifi(&EspSerial);

int temperatureThreshold = 20; // The temperature threshold to start an alert
unsigned long lastUpdated = 0;// last update interval
double temp; //Globally variable for room temperature 
bool emergencyAlert = false;
WidgetTerminal terminal_weather(V9);
String takePill; //Globally variable if the elderly took the pills 
int red = 0, green = 0, blue = 0; //Holds the RGB values sent from the blynk client.
bool receivedRed=false, receivedGreen=false, receivedBlue=false; //Will be true if the RGB values were recieved from Blynk.
int lightsIndex = 0; //Holds the index of the last neopixel led that was lit.
unsigned long lightsOn = 0; //Holds the millis() millisecond of when the neopixel led where last turned on so we can turn them off after 10 seconds.

// This function will be called every time integromat will send the amount of pills
//  to the Virtual Pin V0
BLYNK_WRITE(V0)
{
  int pinValue = param.asInt(); // assigning incoming value from pin V0 to a variable
  for(int i = 0; i < pinValue; ++i){
   CircuitPlayground.setPixelColor(i, rand() % 255,   rand() % 255,   rand() % 255);
  }  
}

// This function will be called every time Button Widget
// in Blynk app writes values to the Virtual Pin V3
BLYNK_WRITE(V3)
{
  int pinValue = param.asInt();// assigning incoming value from pin V3 to a variable  
  takePill = "1";
  CircuitPlayground.clearPixels();
  delay(1000);
  Blynk.virtualWrite(V3, 0); //Initialize the button to No
  Blynk.virtualWrite(V0, 0); //set the number of piils to 0 
 
}

// This function will be called every time integromat will send the  weather temperature
//  to the Virtual Pin V6
BLYNK_WRITE(V6)
{
  Serial.println("Got here");
  double pinValue = param.asDouble(); // assigning incoming value from pin V3 to a variable
  String message = String("The temperature today is ") + pinValue + "℃, ";
  if(pinValue <= 20){
   message = message + "wear a sweather. ";
  }
   Blynk.virtualWrite(V9, message);
}


// This function will be called every time integromat will send the  weather description
//  to the Virtual Pin V7
BLYNK_WRITE(V7)
{
  String pinValue = param.asStr(); // assigning incoming value from pin V3 to a variable
  String message = String("The weather is ") + pinValue;
  Blynk.virtualWrite(V9,message);
}



// This function will be called every time integromat will send 1 if it is rainy
//  to the Virtual Pin V8
BLYNK_WRITE(V8)
{
  int pinValue = param.asInt(); // assigning incoming value from pin V3 to a variable
  if(pinValue == 1){
    Blynk.virtualWrite(V9," .Do not forget to take an umbrella ☂");
  }  
}

// This function will be called every time Slider Widget in Blynk app writes values to the Virtual Pin V5.
//This virtual pin tells the cpx the R value from the RGB value that will be used to light half of the neopixels.
BLYNK_WRITE(V10)
{
   red = param.asInt(); // get a RED channel value
   receivedRed = true;
}

// This function will be called every time Slider Widget in Blynk app writes values to the Virtual Pin V6.
//This virtual pin tells the cpx the G value from the RGB value that will be used to light half of the neopixels.
BLYNK_WRITE(V11)
{
   green = param.asInt(); // get a GREEN channel value
   receivedGreen = true;
}

// This function will be called every time Slider Widget in Blynk app writes values to the Virtual Pin V7.
//This virtual pin tells the cpx the B value from the RGB value that will be used to light half of the neopixels.
BLYNK_WRITE(V12)
{
   blue = param.asInt(); // get a BLUE channel value
   receivedBlue=true;
}


void setup()
{
  SerialUSB.begin(115200);
  CircuitPlayground.begin();

  EspSerial.begin(ESP8266_BAUD);
  delay(10);

  Blynk.begin(auth, wifi, ssid, pass);
  Blynk.virtualWrite(V5, "WE CARE ABOUT YOU"); 
  terminal_weather.clear();
}

void loop()
{
  Blynk.run();
  if(millis() - lastUpdated > 3000) {
   checkValues();
   lastUpdated = millis();
   //Check RGB and set each time half of the leds to the given color.
   checkRGB();
  }
}

void checkValues(){ 
   temp = CircuitPlayground.temperature(); 
   String metrics = String(temp) + " " + takePill; 
   Blynk.virtualWrite(V2, metrics); //send to V2 intragometer the temperature of the room and did he take a pill
   delay(3000);
   takePill = "0"; //set the number of piils to 0
   
   Blynk.virtualWrite(V1, temp); //show temperature on Blynk app
   delay(10);

   //go to energency mode if the temperature is grater then the threshold
   if(temp > temperatureThreshold){ 
    Blynk.virtualWrite(V4, 1); //turn on the led in the Blynk app
    emergencyAlert = true; // alert is on
    emergency(emergencyAlert); //start energency lights
   }
   else{
    Blynk.virtualWrite(V4, 0); //turn off the emergency led on blynk app
    emergencyAlert = false; //turn off the led in the Blynk app
   }
}

//emergency mode, red lights
void emergency(bool on) {
   if(on){
      for(int i = 0; i < 10; ++i){
      CircuitPlayground.setPixelColor(i, 255,   0,   0);
      delay(10);
     }  
    for(int i = 0; i < 10 ; ++i){
      CircuitPlayground.setPixelColor(i, 0,   0,   0);
      delay(10);
     }
    }   
  
}

//This function will check if we recieved all 3 needed values to light the neopixel leds and light half of the neopixels eachtime.
void checkRGB()
{
  //true if we recieved all 3 needed values to light the neopixel leds.
  if(receivedRed && receivedGreen && receivedBlue){
    //Check if we already lit all 10 nepixels and start from the bginning.
    if(lightsIndex==10){
      lightsIndex = 0;
    }
    //light 5 of the neopixels in the given color.
    for (int i = 0 ; i < 5; ++i){
      strip.setPixelColor(lightsIndex, red, green, blue);
      lightsIndex++; 
    }
    strip.show();
    
    lightsOn = millis(); //save the time of the neopixles lighting
    //change the boolean values of the RGB indicators to false so we can recieve the next color.
    receivedRed = false;
    receivedGreen = false;
    receivedBlue = false;
  } 
}
